#include <QtUiTools/QUiLoader>
#include <QtCore/QStringList>
#include <QtCore/QFile>
#include <QtGui/QApplication>
#include <QtGui/QWidget>
#include <QtGui/QListWidget>

#include <Qt3Support/Q3HBox>
#include <Qt3Support/Q3VBox>


#include "uiutils.h"

Q_DECLARE_METATYPE(QListWidget*);

void UiUtils::install( QScriptEngine *engine )
{
    installWidgets( engine );
    installBoxWidgets( engine );

    QScriptValue utilsObject = engine->newObject();

    QScriptValue fun = engine->newFunction( UiUtils::load );
    utilsObject.setProperty("load", fun);
    fun = engine->newFunction( UiUtils::exec );
    utilsObject.setProperty("exec", fun);

    QScriptValue globalObject = engine->globalObject();
    globalObject.setProperty( "UiUtils", utilsObject );

}

void UiUtils::installWidgets( QScriptEngine *engine )
{
    QScriptValue globalObject = engine->globalObject();
    QUiLoader loader;

    QStringList widgets = loader.availableWidgets();
    for ( int i=0; i < widgets.size(); ++i ) {
	QScriptValue fun = engine->newFunction( createWidget );
	QScriptValue name = engine->toScriptValue(widgets[i]);
	fun.setProperty( QString("functionName"), name,
			 QScriptValue::ReadOnly | QScriptValue::Undeletable | QScriptValue::SkipInEnumeration );
	fun.setProperty( QString("prototype"), createPrototype( engine, name.toString() ) );

	globalObject.setProperty(widgets[i], fun);
    }

}

QScriptValue UiUtils::createWidget(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() > 1 )
	return context->throwError("Load takes one argument");

    QWidget *parent = 0;
    if ( context->argumentCount() ) {
	parent = qscriptvalue_cast<QWidget*>(context->argument(0));

	if ( !parent )
	    return context->throwError("The argument to load should be a QWidget");
    }

    QString self = context->callee().property( "functionName" ).toString();
    QUiLoader loader;
    QWidget *w = loader.createWidget( self, parent );

    QScriptValue fun = engine->newQObject( w );
    fun.setPrototype( context->callee().property("prototype") );

    return fun;
}

void UiUtils::installBoxWidgets( QScriptEngine *engine )
{
    QScriptValue fun;
    QScriptValue globalObject = engine->globalObject();

    fun = engine->newFunction( UiUtils::createBox );
    fun.setProperty( QString("functionName"), engine->toScriptValue(QString("QHBox")) );
    globalObject.setProperty("QHBox", fun);

    fun = engine->newFunction( UiUtils::createBox );
    // The following doesn't work, bug!
    // fun.setProperty( QString("functionName"), engine->toScriptValue("QVBox") );
    fun.setProperty( QString("functionName"), engine->toScriptValue(QString("QVBox")) );
    globalObject.setProperty("QVBox", fun);
}

QScriptValue UiUtils::createBox(QScriptContext *context, QScriptEngine *engine)
{
    QWidget *parent = 0;
    if ( context->argumentCount() ) {
	parent = qscriptvalue_cast<QWidget*>(context->argument(0));

	if ( !parent )
	    return context->throwError("The parent should be a QWidget");
    }

    QString self = context->callee().property( "functionName" ).toString();

    QWidget *w;
    if ( self == QString("QHBox") ) {
	w = new Q3HBox(parent);
    }
    else if ( self == QString("QVBox") ) {
	w = new Q3VBox(parent);
    }

    return engine->newQObject( w );
}

QScriptValue UiUtils::load(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() != 1 )
	return context->throwError("Load takes one argument");

    QUiLoader loader;
    QString filename = context->argument(0).toString();
    QFile f( filename );
    if ( !f.open(QIODevice::ReadOnly) )
	return context->throwError(QString("Unable to open '%1'").arg(filename) );

    QWidget *w = loader.load( &f );

    return engine->newQObject( w );
}

QScriptValue UiUtils::exec(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() )
	return context->throwError("exec takes no arguments");

    return engine->toScriptValue( QApplication::exec() );
}

//
// Object bindings
//

QScriptValue UiUtils::createPrototype( QScriptEngine *engine, const QString &name )
{
	QScriptValue proto = engine->newObject();

	if ( name == QString("QListWidget") ) {
	    proto.setProperty( QString("addItem"), engine->newFunction(qlistwidgetAddItem) );
	}

	return proto;
}

QScriptValue UiUtils::qlistwidgetAddItem(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() != 1 )
	return context->throwError("Add item takes one argument");

    // Find our QListWidget
    QListWidget *list = qscriptvalue_cast<QListWidget*>(context->thisObject());
    list->addItem( context->argument(0).toString() );

    return engine->undefinedValue();
}
